@@ -93,7 +93,9 @@ def raise_parsed_exception(api_exception: ApiException):
 
 def wrap_api_client_with_retries(api_client: DefaultApi) -> DefaultApi:
     @wrapt.decorator  # type: ignore
-    def retrier(func: Any, instance: Any, args: List[Any], kwargs: Any) -> Any:
+    def retrier(
+        func: Any, instance: Any, args: List[Any], kwargs: Any  # noqa: ARG001
+    ) -> Any:  # noqa: ARG001
         for i in range(RETRY_COUNT):
             try:
                 return func(*args, **kwargs)
@@ -224,7 +226,7 @@ class AnyscaleInstanceManagerNodeProvider(NodeProvider):
         node_config: Dict[str, Any],
         tags: Dict[str, str],
         count: int,
-        resources: Dict[str, float],
+        resources: Dict[str, float],  # noqa: ARG002
     ) -> None:
         """Mirrors Ray's NodeProvider interface. Needed for compatibility with
         Ray Autoscaler version >=1.8.0
@@ -326,7 +328,7 @@ class AnyscaleInstanceManagerNodeProvider(NodeProvider):
             raise ValueError("IP {ip_address} not found.")
         return cast(str, resp.result.instance_id)
 
-    def get_command_runner(
+    def get_command_runner(  # noqa: PLR0913
         self,
         log_prefix: str,
         node_id: str,
@@ -427,18 +429,20 @@ class AnyscalePoolingNodeProvider(AnyscaleInstanceManagerNodeProvider):
         config_to_hash = copy.deepcopy(node_config)
         if "TagSpecifications" in config_to_hash:
             del config_to_hash["TagSpecifications"]
-        instance_reuse_id = hash_launch_conf(config_to_hash, dict())
+        instance_reuse_id = hash_launch_conf(config_to_hash, {})
         tags[self.INSTANCE_REUSE_TAG] = instance_reuse_id
 
         # Only handle AWS for now.
         # TODO(xcl): add support for non-aws instances.
         bypass_rapid_start = self._get_from_configs("bypass_rapid_start", None)
         # Pooling version (with checks). pool_limit == 0 effectively disables pool.
-        if not (bypass_rapid_start or self._pool_limit() == 0):
-            if self._is_aws_provider():
-                for _ in range(count):
-                    self._create_single_node_aws(node_config, tags)
-                return
+        if (
+            not (bypass_rapid_start or self._pool_limit() == 0)
+            and self._is_aws_provider()
+        ):
+            for _ in range(count):
+                self._create_single_node_aws(node_config, tags)
+            return
 
         # Non-pooling version.
         logger.info("Configuration bypasses Rapid Start; launching node normally.")
@@ -466,7 +470,7 @@ class AnyscalePoolingNodeProvider(AnyscaleInstanceManagerNodeProvider):
         # If we get a node_id back from the instance pool,
         # start this node.
         if node_id is not None:
-            logger.info("Launching node from instance pool! ID {}".format(node_id))
+            logger.info(f"Launching node from instance pool! ID {node_id}")
             super().start_nodes([node_id])
 
             tags[self.POOLED_INSTANCE_TAG] = "true"
@@ -482,7 +486,7 @@ class AnyscalePoolingNodeProvider(AnyscaleInstanceManagerNodeProvider):
                         for tagpair in tagspec["Tags"]:
                             tags[tagpair["Key"]] = tagpair["Value"]
 
-            logger.info("Setting node tags on reused instance: {}".format(str(tags)))
+            logger.info(f"Setting node tags on reused instance: {str(tags)}")
             super().set_node_tags(node_id, tags)
 
         # Otherwise, start a node normally.
@@ -526,7 +530,7 @@ class AnyscalePoolingNodeProvider(AnyscaleInstanceManagerNodeProvider):
         super().stop_nodes([node_id])
         super().wait_until_stopped(node_id)
 
-        logger.info("Sending node {} to the instance pool.".format(node_id))
+        logger.info(f"Sending node {node_id} to the instance pool.")
 
         instance_reuse_id = launch_hash
         instance_pool_submission = AddInstancePoolMember(
@@ -603,12 +607,12 @@ class AnyscaleExecNodeProvider(NodeProvider):  # type: ignore
         super().__init__(provider_config, cluster_name)
         self.dns_address = provider_config["dns_address"]
 
-    def non_terminated_nodes(self, tag_filters: List[str]) -> List[str]:
+    def non_terminated_nodes(self, tag_filters: List[str]) -> List[str]:  # noqa: ARG002
         return [self.dns_address]
 
     def external_ip(self, node_id: str) -> str:
         return node_id
 
-    def node_tags(self, node_id: str) -> Dict[str, Any]:
+    def node_tags(self, node_id: str) -> Dict[str, Any]:  # noqa: ARG002
         # Necessary because Ray requires that head node to have this tag set.
         return {TAG_RAY_NODE_STATUS: STATUS_UP_TO_DATE}
