@@ -112,7 +112,7 @@ def get_last_used_cloud(
     if cloud_id:
         try:
             cloud = anyscale_api_client.get_cloud(cloud_id).result
-        except Exception:
+        except Exception:  # noqa: BLE001
             raise click.ClickException(
                 f"Failed to fetch Cloud with id: {cloud_id}. Please specify `cloud` in the command."
             )
@@ -130,10 +130,8 @@ def get_last_used_cloud(
 
     cloud_name = cloud.name
     log.info(
-        (
-            f"Using last active cloud '{cloud_name}'. "
-            "Specify `cloud` in the command to overwrite."
-        )
+        f"Using last active cloud '{cloud_name}'. "
+        "Specify `cloud` in the command to overwrite."
     )
     return cast(str, cloud_name)
 
