@@ -29,10 +29,10 @@ class ClusterInfo:
 def get_job_submission_client_cluster_info(
     address: str,
     create_cluster_if_needed: Optional[bool] = None,
-    cookies: Optional[Dict[str, Any]] = None,
+    cookies: Optional[Dict[str, Any]] = None,  # noqa: ARG001
     metadata: Optional[Dict[str, Any]] = None,
-    headers: Optional[Dict[str, Any]] = None,
-    **kwargs: Any,
+    headers: Optional[Dict[str, Any]] = None,  # noqa: ARG001
+    **kwargs: Any,  # noqa: ARG001
 ) -> ClusterInfo:
     """
     Get address and cookies used for ray JobSubmissionClient.
