@@ -16,7 +16,7 @@ from anyscale.util import get_working_dir, send_json_request
 @wrapt.decorator  # type: ignore
 def _with_safe_cluster_config(
     wrapped: Callable[..., MutableMapping[str, Any]],
-    instance: Optional[Any],
+    instance: Optional[Any],  # noqa: ARG001
     args: Any,
     kwargs: Any,
 ) -> Any:
@@ -40,7 +40,7 @@ def _configure_cluster_name(
             session_id
         ).result.cluster_name
     else:
-        name = send_json_request("/api/v2/sessions/{}/details".format(session_id), {})[
+        name = send_json_request(f"/api/v2/sessions/{session_id}/details", {})[
             "result"
         ]["cluster_name"]
 
@@ -58,7 +58,7 @@ def _configure_ssh_key(
 ) -> MutableMapping[str, Any]:
     # TODO (yiran): cleanup SSH keys if session no longer exists.
     def _write_ssh_key(name: str, ssh_key: str) -> str:
-        key_path = os.path.join(os.path.expanduser(ssh_dir), "{}.pem".format(name))
+        key_path = os.path.join(os.path.expanduser(ssh_dir), f"{name}.pem")
         os.makedirs(os.path.dirname(key_path), exist_ok=True)
 
         with open(key_path, "w", opener=partial(os.open, mode=0o600)) as f:
@@ -72,9 +72,9 @@ def _configure_ssh_key(
         ).result
         key_path = _write_ssh_key(ssh_key.key_name, ssh_key.private_key)
     else:
-        ssh_key = send_json_request(
-            "/api/v2/sessions/{}/ssh_key".format(session_id), {}
-        )["result"]
+        ssh_key = send_json_request(f"/api/v2/sessions/{session_id}/ssh_key", {})[
+            "result"
+        ]
         key_path = _write_ssh_key(ssh_key["key_name"], ssh_key["private_key"])
 
     cluster_config.setdefault("auth", {})["ssh_private_key"] = key_path
@@ -92,11 +92,11 @@ def _configure_autoscaler_credentials(
         ).result.credentials
     else:
         credentials = send_json_request(
-            "/api/v2/sessions/{}/autoscaler_credentials".format(session_id), {},
+            f"/api/v2/sessions/{session_id}/autoscaler_credentials", {},
         )["result"]["credentials"]
 
     cloud_provider = cluster_config["provider"]["type"]
-    cluster_config["provider"]["{}_credentials".format(cloud_provider)] = credentials
+    cluster_config["provider"][f"{cloud_provider}_credentials"] = credentials
 
     return cluster_config
 
@@ -156,7 +156,7 @@ def configure_for_session(
 
         else:
             _resp: Dict[str, Any] = send_json_request(
-                "/api/v2/sessions/{}/cluster_config".format(session_id), {}
+                f"/api/v2/sessions/{session_id}/cluster_config", {}
             )["result"]
             cluster_config = json.loads(_resp["config_with_defaults"])
 
