@@ -47,7 +47,7 @@ def project_cli() -> None:
     help="Max items to show in list.",
     callback=validate_non_negative_arg,
 )
-def list(
+def list(  # noqa: A001
     name: str,
     json: bool,
     created_by_me: bool,
@@ -64,7 +64,7 @@ def list(
     project_controller.list(name, json, created_by_me, max_items)
 
 
-def _validate_project_name(ctx, param, value) -> str:
+def _validate_project_name(ctx, param, value) -> str:  # noqa: ARG001
     if value and not validate_project_name(value):
         raise click.BadParameter(
             '"{}" contains spaces. Please enter a project name without spaces'.format(
