@@ -95,8 +95,8 @@ def _get_base_image(image: str, ray_version: str, cpu_or_gpu: str) -> str:
     if sys.version_info.major == 3 and sys.version_info.minor == 10:
         py_version = "310"
     if py_version not in ["36", "37", "38", "39", "310"]:
-        raise ValueError("No default docker image for py{}".format(py_version))
-    return "anyscale/{}:{}-py{}-{}".format(image, ray_version, py_version, cpu_or_gpu)
+        raise ValueError(f"No default docker image for py{py_version}")
+    return f"anyscale/{image}:{ray_version}-py{py_version}-{cpu_or_gpu}"
 
 
 def _is_in_shell(frames: Optional[List[Any]] = None) -> bool:
@@ -121,7 +121,7 @@ def _is_running_on_anyscale_cluster() -> bool:
 
 
 def _redact_token(token: str) -> str:
-    "Preserve a short prefix of the token, redact other characters."
+    """Preserve a short prefix of the token, redact other characters."""
     preserve = 8
     n = len(token)
     if n <= preserve:
@@ -164,7 +164,7 @@ class ClientBuilder:
         >>> [0, 1, 4, 9, 16]
     """
 
-    def __init__(
+    def __init__(  # noqa: PLR0913, PLR0915
         self,
         address: Optional[str] = None,
         anyscale_sdk: AnyscaleSDK = None,
@@ -196,7 +196,7 @@ class ClientBuilder:
                 )
             )
             redacted_token = _redact_token(self._credentials)
-            self._log.debug("Using credentials {}".format(redacted_token))
+            self._log.debug(f"Using credentials {redacted_token}")
             self._anyscale_sdk = AnyscaleSDK(
                 self._credentials,
                 anyscale.shared_anyscale_utils.conf.ANYSCALE_HOST + "/ext",
@@ -353,7 +353,7 @@ class ClientBuilder:
                     f"{key}. The supported parameters are: {CONNECT_URL_PARAMS}."
                 )
 
-    def _init_args(self, **kwargs) -> "ClientBuilder":  # noqa: C901
+    def _init_args(self, **kwargs) -> "ClientBuilder":  # noqa: C901, PLR0912
         """
         Accepts arguments from ray.init when called with anyscale protocol,
         i.e. ray.init("anyscale://someCluster", arg1="thing", arg2="other").
@@ -617,7 +617,7 @@ class ClientBuilder:
                     raise ValueError(f"Can't find conda YAML file {yaml_file}.")
                 try:
                     runtime_env["conda"] = yaml.safe_load(yaml_file.read_text())
-                except Exception as e:
+                except Exception as e:  # noqa: BLE001
                     raise ValueError(f"Failed to read conda file {yaml_file}: {e}.")
 
     def _pin_protobuf_in_runtime_env_if_needed(
@@ -638,12 +638,18 @@ class ClientBuilder:
         # https://github.com/ray-project/ray/commit/6c8eb5e2ebde8db213ffb8722a0324077188e308
         if version.parse(self._ray.__version__) < version.parse("1.13.0"):
             self._rewrite_runtime_env_conda_as_dict(runtime_env)
-            if "conda" in runtime_env and isinstance(runtime_env["conda"], dict):
-                if "dependencies" in runtime_env["conda"]:
-                    for dep in runtime_env["conda"]["dependencies"]:
-                        if isinstance(dep, dict) and "pip" in dep:
-                            if isinstance(dep["pip"], list):
-                                dep["pip"].append("protobuf==3.20.1")
+            if (
+                "conda" in runtime_env
+                and isinstance(runtime_env["conda"], dict)
+                and "dependencies" in runtime_env["conda"]
+            ):
+                for dep in runtime_env["conda"]["dependencies"]:
+                    if (
+                        isinstance(dep, dict)
+                        and "pip" in dep
+                        and isinstance(dep["pip"], list)
+                    ):
+                        dep["pip"].append("protobuf==3.20.1")
 
     def _set_runtime_env_in_job_config(self, project_dir: Optional[str]) -> None:
         """Configures the runtime env inside self._job_config.
@@ -660,10 +666,9 @@ class ClientBuilder:
             else []
         )
 
-        if "working_dir" not in runtime_env:
-            if project_dir:
-                # TODO(nikita): Remove logic of implying working dir from project dir
-                runtime_env["working_dir"] = project_dir
+        if "working_dir" not in runtime_env and project_dir:
+            # TODO(nikita): Remove logic of implying working dir from project dir
+            runtime_env["working_dir"] = project_dir
         if "excludes" not in runtime_env:
             runtime_env["excludes"] = []
         runtime_env["excludes"] = (
@@ -783,7 +788,7 @@ class ClientBuilder:
             self._user_runtime_env.setdefault("env_vars", {}).setdefault(
                 SERVE_ROOT_URL_ENV_KEY, cluster.user_service_url.rstrip("/")
             )
-        except Exception:
+        except Exception:  # noqa: BLE001
             self._log.warning(
                 f"Unable to set {SERVE_ROOT_URL_ENV_KEY} in runtime environment. Please specify "
                 "full Serve session URL for Serve deployments."
@@ -974,14 +979,13 @@ class ClientBuilder:
             >>> ray.init("anyscale://cluster_name")
         """
         if run_mode not in [None, "local_docker"]:
-            raise ValueError("Unknown run mode {}".format(run_mode))
-        if self._in_shell:
-            if run_mode == "local_docker":
-                raise ValueError("Local docker mode is not supported in Python shells.")
+            raise ValueError(f"Unknown run mode {run_mode}")
+        if self._in_shell and run_mode == "local_docker":
+            raise ValueError("Local docker mode is not supported in Python shells.")
         self._run_mode = run_mode
         return self
 
-    def base_docker_image(self, image_name: str) -> None:
+    def base_docker_image(self, image_name: str) -> None:  # noqa: ARG002
         """[DEPRECATED] Set the docker image to use for the cluster.
         IMPORTANT: the Python minor version of the manually specified docker
         image must match the local Python version.
@@ -996,7 +1000,7 @@ class ClientBuilder:
             '`ray.init("anyscale://cluster_name?cluster_env=name:1")`'
         )
 
-    def require(self, requirements: Union[str, List[str]]) -> None:
+    def require(self, requirements: Union[str, List[str]]) -> None:  # noqa: ARG002
         """[DEPRECATED] Set the Python requirements for the cluster.
         Args:
             requirements: either be a list of pip library specifications, or
@@ -1274,7 +1278,7 @@ class ClientBuilder:
             ).remote()
             cluster_id = self._ray.get(ray_id)
             cluster_name = self._anyscale_sdk.get_session(cluster_id).result.name
-        except Exception as e:
+        except Exception as e:  # noqa: BLE001
             self._log.error(f"Failed to get ClusterID with error: {e}")
             raise RuntimeError(
                 "Unable to get ClusterID for `download_results`."
@@ -1285,7 +1289,7 @@ class ClientBuilder:
         """Query Anyscale for the given cluster's metadata."""
         cluster_found = get_cluster(self._anyscale_sdk, project_id, session_name)
         if not cluster_found:
-            raise RuntimeError("Failed to locate cluster: {}".format(session_name))
+            raise RuntimeError(f"Failed to locate cluster: {session_name}")
         return cluster_found
 
     def _exec_self_in_head_node(self) -> None:
@@ -1293,7 +1297,7 @@ class ClientBuilder:
         cur_file = os.path.abspath(sys.argv[0])
         # TODO(ekl) it would be nice to support keeping the original file name,
         # but "anyscale push" isn't escaping e.g., spaces correctly.
-        tmp_file = "/tmp/anyscale-connect-{}.py".format(uuid.uuid4().hex)
+        tmp_file = f"/tmp/anyscale-connect-{uuid.uuid4().hex}.py"
         cur_dir = os.getcwd()
         try:
             assert self._project_dir is not None
@@ -1333,9 +1337,9 @@ class ClientBuilder:
                 anyscale.shared_anyscale_utils.conf.ANYSCALE_HOST
             ),
             "--env",
-            "ANYSCALE_CLI_TOKEN={}".format(self._credentials),
+            f"ANYSCALE_CLI_TOKEN={self._credentials}",
             "-v",
-            "{}:/user_main.py".format(cur_file),
+            f"{cur_file}:/user_main.py",
             "--entrypoint=/bin/bash",
             docker_image,
             "-c",
