@@ -36,8 +36,12 @@ from anyscale.client.openapi_client.models import (
     WriteCloud,
 )
 from anyscale.client.openapi_client.models.cloud_state import CloudState
+from anyscale.client.openapi_client.models.subnet_id_with_availability_zone_aws import (
+    SubnetIdWithAvailabilityZoneAWS,
+)
 from anyscale.cloud import get_cloud_id_and_name, get_cloud_json_from_id
 from anyscale.cloud_resource import (
+    associate_subnets_with_azs,
     verify_aws_cloudformation_stack,
     verify_aws_efs,
     verify_aws_iam_roles,
@@ -53,6 +57,7 @@ from anyscale.shared_anyscale_utils.aws import AwsRoleArn, get_dataplane_role_ar
 from anyscale.shared_anyscale_utils.conf import ANYSCALE_ENV
 from anyscale.util import (  # pylint:disable=private-import
     _client,
+    _get_aws_efs_mount_target_ip,
     _get_role,
     _resource,
     _update_external_ids_for_policy,
@@ -83,8 +88,11 @@ DEFAULT_RAY_IAM_ROLE = RAY + "-v1"
 
 class CloudController(BaseController):
     def __init__(
-        self, log: LogsLogger = LogsLogger(), initialize_auth_api_client: bool = True
+        self, log: Optional[LogsLogger] = None, initialize_auth_api_client: bool = True
     ):
+        if log is None:
+            log = LogsLogger()
+
         super().__init__(initialize_auth_api_client=initialize_auth_api_client)
         self.log = log
         self.log.open_block("Output")
@@ -163,7 +171,7 @@ class CloudController(BaseController):
                 False,
             )
 
-    def setup_cloud(
+    def setup_cloud(  # noqa: PLR0913
         self,
         provider: str,
         region: Optional[str],
@@ -244,7 +252,7 @@ class CloudController(BaseController):
         region: str,
         cloud_id: str,
         anyscale_iam_role_name: str,
-        cluster_node_iam_role_arn: str,
+        cluster_node_iam_role_name: str,
     ) -> Dict[str, Any]:
         response = (
             self.api_client.get_anyscale_aws_account_api_v2_clouds_anyscale_aws_account_get()
@@ -288,11 +296,11 @@ class CloudController(BaseController):
                     ),
                 },
                 {
-                    "ParameterKey": "ClusterNodeIAMRoleArn",
-                    "ParameterValue": cluster_node_iam_role_arn,
+                    "ParameterKey": "ClusterNodeIAMRoleName",
+                    "ParameterValue": cluster_node_iam_role_name,
                 },
             ],
-            Capabilities=["CAPABILITY_NAMED_IAM"],
+            Capabilities=["CAPABILITY_NAMED_IAM", "CAPABILITY_AUTO_EXPAND"],
         )
 
         stacks = cfn_client.describe_stacks(StackName=cfn_stack_name)
@@ -328,9 +336,7 @@ class CloudController(BaseController):
                 )
         return cfn_stack
 
-    def update_cloud_with_resources(
-        self, cfn_stack: Dict[str, Any], cloud_id: str, cluster_node_iam_role_arn: str
-    ):
+    def update_cloud_with_resources(self, cfn_stack: Dict[str, Any], cloud_id: str):
         if "Outputs" not in cfn_stack:
             raise ClickException(
                 f"Timed out setting up cloud resources. Please check your cloudformation stack for errors. {cfn_stack['StackId']}"
@@ -345,17 +351,26 @@ class CloudController(BaseController):
             ), f"{resource_type} is not created properly. Please delete the cloud and try creating agian."
             cfn_resources[resource_type] = resource_value
 
-        aws_subnet_ids = cfn_resources["PublicSubnets"].split(",")
+        aws_subnets_with_availability_zones = json.loads(
+            cfn_resources["SubnetsWithAvailabilityZones"]
+        )
         aws_vpc_id = cfn_resources["VPC"]
         aws_security_groups = [cfn_resources["AnyscaleSecurityGroup"]]
         aws_s3_id = cfn_resources["S3Bucket"]
         aws_efs_id = cfn_resources["EFS"]
         aws_efs_mount_target_ip = cfn_resources["EFSMountTargetIP"]
         anyscale_iam_role_arn = cfn_resources["AnyscaleIAMRole"]
+        cluster_node_iam_role_arn = cfn_resources["NodeIamRole"]
 
         create_cloud_resource = CreateCloudResource(
             aws_vpc_id=aws_vpc_id,
-            aws_subnet_ids=aws_subnet_ids,
+            aws_subnet_ids_with_availability_zones=[
+                SubnetIdWithAvailabilityZoneAWS(
+                    subnet_id=subnet_with_az["subnet_id"],
+                    availability_zone=subnet_with_az["availability_zone"],
+                )
+                for subnet_with_az in aws_subnets_with_availability_zones
+            ],
             aws_iam_role_arns=[anyscale_iam_role_arn, cluster_node_iam_role_arn],
             aws_security_groups=aws_security_groups,
             aws_s3_id=aws_s3_id,
@@ -373,7 +388,7 @@ class CloudController(BaseController):
 
     def prepare_for_managed_cloud_setup(
         self, region: str, cloud_name: str
-    ) -> Tuple[str, str, str]:
+    ) -> Tuple[str, str]:
         regions_available = get_available_regions()
         if region not in regions_available:
             raise ClickException(
@@ -407,22 +422,15 @@ class CloudController(BaseController):
             )
         ).result
 
-        # Handle the creation of ray autoscaler role in python for now because this role could already exist in customer account
-        # and it's not extremely easy to do conditionals (create if not exist already) in cloudformation.
-        # TODO: Move this to be created by cloudformation
-        cluster_node_iam_role_arn = self.setup_aws_ray_role(
-            region, f"{created_cloud.id}-cluster_node_role"
-        )
+        return anyscale_iam_role_name, created_cloud.id
 
-        return anyscale_iam_role_name, cluster_node_iam_role_arn.arn, created_cloud.id
-
-    def setup_managed_cloud(
+    def setup_managed_cloud(  # noqa: PLR0913
         self,
         provider: str,
         region: str,
         name: str,
-        yes: bool = False,
-        folder_id: Optional[int] = None,
+        yes: bool = False,  # noqa: ARG002
+        folder_id: Optional[int] = None,  # noqa: ARG002
     ) -> None:
         """
         Sets up a cloud provider
@@ -431,18 +439,18 @@ class CloudController(BaseController):
             with self.log.spinner("Preparing environment for cloud setup..."):
                 (
                     anyscale_iam_role_name,
-                    cluster_node_iam_role_arn,
                     cloud_id,
                 ) = self.prepare_for_managed_cloud_setup(region, name)
 
             try:
                 cfn_stack = self.run_cloudformation(
-                    region, cloud_id, anyscale_iam_role_name, cluster_node_iam_role_arn
+                    region,
+                    cloud_id,
+                    anyscale_iam_role_name,
+                    f"{cloud_id}-cluster_node_role",
                 )
-                self.update_cloud_with_resources(
-                    cfn_stack, cloud_id, cluster_node_iam_role_arn
-                )
-            except Exception as e:
+                self.update_cloud_with_resources(cfn_stack, cloud_id)
+            except Exception as e:  # noqa: BLE001
                 self.log.error(str(e))
                 self.api_client.delete_cloud_api_v2_clouds_cloud_id_delete(
                     cloud_id=cloud_id
@@ -483,9 +491,9 @@ class CloudController(BaseController):
             try:
                 role = _get_role(role_name=role_name, region=region)
                 if role:
-                    for policy in role.policies.all():
+                    for policy in role.policies.all():  # type: ignore
                         policy.delete()
-                    role.delete()
+                    role.delete()  # type: ignore
             except botocore.exceptions.ClientError:
                 self.log.info(
                     f"Failed to delete IAM role during cloud deletion: {role_name}"
@@ -524,9 +532,9 @@ class CloudController(BaseController):
         )
 
         for _ in range(5):
-            role_name = "{}-{}".format(ANYSCALE_IAM_ROLE_NAME, secrets.token_hex(4))
+            role_name = f"{ANYSCALE_IAM_ROLE_NAME}-{secrets.token_hex(4)}"
 
-            role = _get_role(role_name, region)
+            role: Any = _get_role(role_name, region)
             if role is None:
                 break
         else:
@@ -534,7 +542,7 @@ class CloudController(BaseController):
                 "We weren't able to connect your account with the Anyscale because we weren't able to find an available IAM Role name in your account. Please reach out to support or your SA for assistance."
             )
 
-        iam = _resource("iam", region)
+        iam: Any = _resource("iam", region)
         iam.create_role(
             RoleName=role_name,
             AssumeRolePolicyDocument=json.dumps(anyscale_aws_iam_role_policy),
@@ -560,7 +568,7 @@ class CloudController(BaseController):
                 )
             )
         except ClickException:
-            self.log.info("Create failed, cleaning up IAM role: {}".format(role_name))
+            self.log.info(f"Create failed, cleaning up IAM role: {role_name}")
             try:
                 for policy in role.policies.all():
                     policy.delete()
@@ -594,7 +602,7 @@ class CloudController(BaseController):
     def setup_aws_ray_role(self, region: str, role_name: str) -> Any:
         iam = boto3.resource("iam", region_name=region)
 
-        role = _get_role(role_name, region)
+        role: Any = _get_role(role_name, region)
         if role is None:
             iam.create_role(
                 RoleName=role_name,
@@ -602,6 +610,7 @@ class CloudController(BaseController):
             )
 
             role = _get_role(role_name, region)
+        assert role is not None, f"Role: {role_name} could not be created"
 
         role.attach_policy(PolicyArn=AMAZON_ECR_READONLY_ACCESS_POLICY_ARN)
         # Modified permissions from Ray (no EC2FullAccess)
@@ -745,7 +754,7 @@ class CloudController(BaseController):
                 show_default=True,
             )
 
-        role = _get_role(role_name, cloud.region)
+        role: Any = _get_role(role_name, cloud.region)
         assert (
             role is not None
         ), f"Failed to find IAM role {role_name} in Cloud {cloud.name}! Have you run 'cloud setup'?"
@@ -869,12 +878,12 @@ class CloudController(BaseController):
             .execute()
         )
         if api_state["state"] != "ENABLED":
-            if not yes:
-                if not click.confirm(
-                    f"The project projects/{project_name} doesn't have the Secret Manager "
-                    "API enabled. Do you want to enable it?"
-                ):
-                    return
+            if not yes and not click.confirm(
+                f"The project projects/{project_name} doesn't have the Secret Manager "
+                "API enabled. Do you want to enable it?"
+            ):
+                return
+
             (
                 serviceusage_resource.services()
                 .enable(
@@ -890,7 +899,12 @@ class CloudController(BaseController):
     def _passed_or_failed_str_from_bool(self, is_passing: bool) -> str:
         return "PASSED" if is_passing else "FAILED"
 
-    def verify_cloud(self, cloud_name: Optional[str], cloud_id: Optional[str]) -> bool:
+    def verify_cloud(
+        self,
+        cloud_name: Optional[str],
+        cloud_id: Optional[str],
+        boto3_session: Optional[Any] = None,
+    ) -> bool:
         """
         Verifies a cloud by name or id.
         """
@@ -915,7 +929,8 @@ class CloudController(BaseController):
             return False
 
         if cloud.provider == "AWS":
-            boto3_session = boto3.Session(region_name=cloud.region)
+            if boto3_session is None:
+                boto3_session = boto3.Session(region_name=cloud.region)
             return self.verify_aws_cloud_resources(
                 cloud_resource=cloud.cloud_resource,
                 boto3_session=boto3_session,
@@ -935,10 +950,10 @@ class CloudController(BaseController):
             )
             return False
 
-    def verify_aws_cloud_resources(
+    def verify_aws_cloud_resources(  # noqa: PLR0913
         self,
         cloud_resource: CreateCloudResource,
-        boto3_session: Any,
+        boto3_session: boto3.Session,
         region: str,
         is_private_network: bool,
         is_bring_your_own_resource: bool = False,
@@ -950,22 +965,20 @@ class CloudController(BaseController):
             logger=self.log,
             ignore_capacity_errors=ignore_capacity_errors,
         )
-        verify_aws_subnets_result, reordered_cloud_resource = verify_aws_subnets(
+        verify_aws_subnets_result = verify_aws_subnets(
             cloud_resource=cloud_resource,
             region=region,
             logger=self.log,
             ignore_capacity_errors=ignore_capacity_errors,
             is_private_network=is_private_network,
         )
-        if reordered_cloud_resource:
-            cloud_resource = reordered_cloud_resource
+
         anyscale_aws_account = (
             self.api_client.get_anyscale_aws_account_api_v2_clouds_anyscale_aws_account_get().result.anyscale_aws_account
         )
         verify_aws_iam_roles_result = verify_aws_iam_roles(
             cloud_resource=cloud_resource,
             boto3_session=boto3_session,
-            region=region,
             anyscale_aws_account=anyscale_aws_account,
             logger=self.log,
         )
@@ -1016,7 +1029,7 @@ class CloudController(BaseController):
             ]
         )
 
-    def register_aws_cloud(
+    def register_aws_cloud(  # noqa: PLR0913
         self,
         region: str,
         name: str,
@@ -1058,12 +1071,16 @@ class CloudController(BaseController):
                 role is not None
             ), f"Failed to access IAM role {anyscale_iam_role_id}."
             new_policy = _update_external_ids_for_policy(
-                role.assume_role_policy_document, cloud_id
+                role.assume_role_policy_document, cloud_id  # type: ignore
+            )
+            role.AssumeRolePolicy().update(PolicyDocument=json.dumps(new_policy))  # type: ignore
+
+            boto3_session = boto3.Session(region_name=region)
+            aws_efs_mount_target_ip = _get_aws_efs_mount_target_ip(
+                boto3_session, efs_id
             )
-            role.AssumeRolePolicy().update(PolicyDocument=json.dumps(new_policy))
 
             # Verify cloud resources meet our requirement
-            boto3_session = boto3.Session(region_name=region)
             create_cloud_resource = CreateCloudResource(
                 aws_vpc_id=vpc_id,
                 aws_subnet_ids=subnet_ids,
@@ -1071,6 +1088,7 @@ class CloudController(BaseController):
                 aws_security_groups=security_group_ids,
                 aws_s3_id=s3_bucket_id,
                 aws_efs_id=efs_id,
+                aws_efs_mount_target_ip=aws_efs_mount_target_ip,
             )
 
             with self.log.spinner("Verifying cloud resources..."):
@@ -1085,6 +1103,9 @@ class CloudController(BaseController):
                         "Cloud registration failed! Please make sure all the resources provided meet the requirements and try again."
                     )
 
+                create_cloud_resource = associate_subnets_with_azs(
+                    create_cloud_resource, region
+                )
                 # update cloud with verified cloud resources
                 self.api_client.update_cloud_with_cloud_resource_api_v2_clouds_with_cloud_resource_router_cloud_id_put(
                     cloud_id=cloud_id,
@@ -1092,7 +1113,7 @@ class CloudController(BaseController):
                         cloud_resource_to_update=create_cloud_resource,
                     ),
                 )
-        except Exception as e:
+        except Exception as e:  # noqa: BLE001
             # Delete the cloud if registering the cloud fails
             self.api_client.delete_cloud_api_v2_clouds_cloud_id_delete(
                 cloud_id=cloud_id
@@ -1116,7 +1137,7 @@ class CloudController(BaseController):
         )
 
         confirm(
-            f"You'll lose access to existing sessions created with cloud {cloud_id} if you drop it.\nContinue?",
+            f"If the cloud {cloud_id} is deleted, you will not be able to access existing clusters of this cloud.\nFor more information, please refer to the documentation https://docs.anyscale.com/user-guide/onboard/clouds#cloud-deletion\nContinue?",
             skip_confirmation,
         )
 
@@ -1169,33 +1190,23 @@ class CloudController(BaseController):
             )
 
         cfn_client = _client("cloudformation", cloud.region)
-        cfn_stack_name = cloud.cloud_resource.aws_cloudformation_stack_id.split("/")[-2]
-        cfn_client.delete_stack(StackName=cfn_stack_name)
+        cfn_stack_arn = cloud.cloud_resource.aws_cloudformation_stack_id
+        cfn_client.delete_stack(StackName=cfn_stack_arn)
 
-        cfn_stack = cfn_client.describe_stacks(StackName=cfn_stack_name)["Stacks"][0]
-        cfn_stack_url = f"https://{cloud.region}.console.aws.amazon.com/cloudformation/home?region={cloud.region}#/stacks/stackinfo?stackId={cfn_stack['StackId']}"
+        cfn_stack_url = f"https://{cloud.region}.console.aws.amazon.com/cloudformation/home?region={cloud.region}#/stacks/stackinfo?stackId={cfn_stack_arn}"
         self.log.info(f"\nTrack progress of cloudformation at {cfn_stack_url}")
         with self.log.spinner("Deleting cloud resources through cloudformation..."):
             end_time = time.time() + CLOUDFORMATION_TIMEOUT_SECONDS
             while time.time() < end_time:
-                try:
-                    cfn_stack = cfn_client.describe_stacks(StackName=cfn_stack_name)[
-                        "Stacks"
-                    ][0]
-                except botocore.exceptions.ClientError as e:
-                    if cloud.cloud_resource.aws_cloudformation_stack_id in [
-                        stack["StackId"]
-                        for stack in cfn_client.list_stacks(
-                            StackStatusFilter=["DELETE_COMPLETE"]
-                        )["StackSummaries"]
-                    ]:
-                        self.log.info(
-                            f"Cloudformation stack {cfn_stack['StackId']} is deleted."
-                        )
-                        break
-                    raise ClickException(
-                        f"Describing cloudformation stack failed with {e}. Please check your cloudformation stack for errors. {cfn_stack_url}"
+                cfn_stack = cfn_client.describe_stacks(StackName=cfn_stack_arn)[
+                    "Stacks"
+                ][0]
+
+                if cfn_stack["StackStatus"] == "DELETE_COMPLETE":
+                    self.log.info(
+                        f"Cloudformation stack {cfn_stack['StackId']} is deleted."
                     )
+                    break
 
                 if cfn_stack["StackStatus"] in ("DELETE_FAILED",):
                     # Provide link to cloudformation
@@ -1208,4 +1219,8 @@ class CloudController(BaseController):
                 raise ClickException(
                     f"Timed out deleting AWS resources. Please check your cloudformation stack for errors. {cfn_stack['StackId']}"
                 )
+        self.log.info(
+            f"\nThe S3 bucket ({cloud.cloud_resource.aws_s3_id}) associated with this cloud still exists."
+            "\nIf you no longer need the data associated with this bucket, please delete it."
+        )
         return True
