@@ -5,7 +5,6 @@ from typing import Any, Dict, Optional, Tuple
 import click
 import tabulate
 import yaml
-from yaml.loader import SafeLoader
 
 from anyscale.cli_logger import BlockLogger
 from anyscale.cluster_compute import (
@@ -51,13 +50,16 @@ from anyscale.util import (
 
 class ClusterController(BaseController):
     def __init__(
-        self, log: BlockLogger = BlockLogger(), initialize_auth_api_client: bool = True
+        self, log: Optional[BlockLogger] = None, initialize_auth_api_client: bool = True
     ):
+        if log is None:
+            log = BlockLogger()
+
         super().__init__(initialize_auth_api_client=initialize_auth_api_client)
         self.log = log
         self.log.open_block("Output")
 
-    def start(
+    def start(  # noqa: PLR0913
         self,
         cluster_name: Optional[str],
         cluster_id: Optional[str],
@@ -260,7 +262,7 @@ class ClusterController(BaseController):
         self.anyscale_api_client.archive_cluster(cluster.id)
         self.log.info(f"Cluster {cluster.id} is successfully archived.")
 
-    def list(
+    def list(  # noqa: PLR0912, PLR0913
         self,
         cluster_name: Optional[str],
         cluster_id: Optional[str],
@@ -392,7 +394,7 @@ class ClusterController(BaseController):
                 cluster_name = cluster.name
                 project_id = cluster.project_id
                 assert project_id and cluster_name  # For mypy
-            except Exception:
+            except Exception:  # noqa: BLE001
                 raise click.ClickException(f"No cluster exists with id {cluster_id}.")
         else:
             project_id = get_and_validate_project_id(
@@ -421,7 +423,7 @@ class ClusterController(BaseController):
         ).result
         return cluster_env, build
 
-    def _get_cluster_compute(
+    def _get_cluster_compute(  # noqa: PLR0913
         self,
         cluster_compute_name: Optional[str],
         cluster_compute_file: Optional[str],
@@ -434,10 +436,10 @@ class ClusterController(BaseController):
                 cluster_compute_name, self.api_client
             )
         elif cluster_compute_file:
-            with open(cluster_compute_file, "r") as f:
-                config_dict: Dict[str, Any] = yaml.load(f.read(), Loader=SafeLoader)
+            with open(cluster_compute_file) as f:
+                config_dict: Dict[str, Any] = yaml.safe_load(f)
             cluster_compute_config = ClusterComputeConfig(**config_dict)
-            name = "cli-config-{}".format(datetime.now().isoformat())
+            name = f"cli-config-{datetime.now().isoformat()}"
             cluster_compute = self.anyscale_api_client.create_cluster_compute(
                 CreateClusterCompute(name=name, config=cluster_compute_config)
             ).result
@@ -468,7 +470,7 @@ class ClusterController(BaseController):
         assert cluster_name, f"Cluster name {cluster_name} is invalid."
         return cluster_name
 
-    def _create_or_update_cluster_data(
+    def _create_or_update_cluster_data(  # noqa: PLR0913
         self,
         existing_cluster: Optional[Cluster],
         cluster_name: str,
@@ -523,7 +525,7 @@ class ClusterController(BaseController):
 
         return (cluster, needs_start)
 
-    def _check_needs_start(
+    def _check_needs_start(  # noqa: PLR0913
         self,
         current_cluster: Cluster,
         build: ClusterEnvironmentBuild,
