@@ -83,8 +83,11 @@ class ConfigController(BaseController):
     """
 
     def __init__(
-        self, log: BlockLogger = BlockLogger(), initialize_auth_api_client: bool = True
+        self, log: Optional[BlockLogger] = None, initialize_auth_api_client: bool = True
     ):
+        if log is None:
+            log = BlockLogger()
+
         super().__init__(initialize_auth_api_client=initialize_auth_api_client)
         self.sdk_client = self.anyscale_api_client
 
@@ -118,7 +121,7 @@ class ConfigController(BaseController):
             {"name": name, "config": cluster_compute_config, "anonymous": anonymous}
         )
 
-    def setup_dev_ray(
+    def setup_dev_ray(  # noqa: PLR0913
         self,
         working_dir: str,
         cluster_name: str,
@@ -373,7 +376,7 @@ class ConfigController(BaseController):
 
 # TODO(tchordia) once this function is in prod, remove last dependency from anyscale.background
 def run_kill_child(
-    *popenargs, input=None, timeout=None, check=False, **kwargs
+    *popenargs, input=None, timeout=None, check=False, **kwargs  # noqa: A002
 ) -> subprocess.CompletedProcess:
     """
     This function is a fork of subprocess.run with fewer args.
@@ -387,7 +390,7 @@ def run_kill_child(
     Arguments are the same as subprocess.run
     """
     # Start new session ensures that this subprocess starts as a new process group
-    with subprocess.Popen(start_new_session=True, *popenargs, **kwargs) as process:
+    with subprocess.Popen(*popenargs, start_new_session=True, **kwargs) as process:
         parent_pid = os.getpid()
         child_pid = process.pid
         child_pgid = os.getpgid(child_pid)
