@@ -3,7 +3,7 @@ from shlex import quote
 import subprocess
 import sys
 import tempfile
-from typing import Any, List, Tuple
+from typing import Any, List, Optional, Tuple
 
 import click
 
@@ -34,12 +34,15 @@ def rsync(*args: Any, **kwargs: Any) -> None:
 
 class ExecController(BaseController):
     def __init__(
-        self, log: BlockLogger = BlockLogger(), initialize_auth_api_client: bool = True
+        self, log: Optional[BlockLogger] = None, initialize_auth_api_client: bool = True
     ):
+        if log is None:
+            log = BlockLogger()
+
         super().__init__(initialize_auth_api_client=initialize_auth_api_client)
         self.log = log
 
-    def anyscale_exec(
+    def anyscale_exec(  # noqa: PLR0913
         self,
         session_name: str,
         screen: bool,
@@ -150,7 +153,7 @@ class ExecController(BaseController):
 
         return session.name, session.id
 
-    def _generate_remote_command(
+    def _generate_remote_command(  # noqa: PLR0913
         self,
         session_command_id: str,
         commands: List[str],
