@@ -59,8 +59,11 @@ def rsync(*args: Any, **kwargs: Any) -> None:
 
 class SessionController(BaseController):
     def __init__(
-        self, log: BlockLogger = BlockLogger(), initialize_auth_api_client: bool = True
+        self, log: Optional[BlockLogger] = None, initialize_auth_api_client: bool = True
     ):
+        if log is None:
+            log = BlockLogger()
+
         super().__init__(initialize_auth_api_client=initialize_auth_api_client)
         self.log = log
 
@@ -104,7 +107,7 @@ class SessionController(BaseController):
             f"Session {session_names_str} shutting down. View progress at {url}"
         )
 
-    def start(
+    def start(  # noqa: PLR0913
         self,
         session_name: Optional[str],
         build_id: Optional[str],
@@ -154,7 +157,7 @@ class SessionController(BaseController):
         url = get_endpoint(f"/projects/{project_id}/clusters/{session.id}")
         print(f"Session {session_name} finished starting. View at {url}")
 
-    def ssh(
+    def ssh(  # noqa: PLR0912, PLR0913, PLR0915
         self,
         session_name: str,
         ssh_option: Sequence[str],
@@ -190,7 +193,7 @@ class SessionController(BaseController):
             # be installed.
             ray = try_import_ray()
 
-            with open(config_path, "r") as f:
+            with open(config_path) as f:
                 config_dict = yaml.safe_load(f)
 
             try:
@@ -221,7 +224,7 @@ class SessionController(BaseController):
                 ["ssh"]
                 + list(ssh_option)
                 + ["-tt", "-i", ssh_key_path]
-                + ["{}@{}".format(ssh_user, node_ip)]
+                + [f"{ssh_user}@{node_ip}"]
                 + [f"docker exec -it {container_name} sh -c 'which bash && bash || sh'"]
             )
         else:
@@ -253,7 +256,7 @@ class SessionController(BaseController):
                 ["ssh"]
                 + list(ssh_option)
                 + ["-tt", "-i", key_path]
-                + ["{}@{}".format(ssh_user, head_ip)]
+                + [f"{ssh_user}@{head_ip}"]
             )
             if cmd:
                 command = shlex.quote(cmd)
@@ -288,7 +291,7 @@ class SessionController(BaseController):
 
         subprocess.run(ssh_command)  # noqa: B1
 
-    def pull(
+    def pull(  # noqa: PLR0913
         self,
         session_name: Optional[str] = None,
         source: Optional[str] = None,
@@ -344,16 +347,16 @@ class SessionController(BaseController):
 
             self.log.info("Pull completed.")
 
-        except Exception as e:
+        except Exception as e:  # noqa: BLE001
             print(e)
             raise click.ClickException(str(e))  # type: ignore
 
-    def push(
+    def push(  # noqa: PLR0913
         self,
         session_name: Optional[str],
         source: Optional[str],
         target: Optional[str],
-        config: Optional[str],
+        config: Optional[str],  # noqa: ARG002
         all_nodes: bool,
         rsync_exclude_override: Optional[List[str]] = None,
     ) -> None:
@@ -425,12 +428,12 @@ class SessionController(BaseController):
         """
         Register the compute config with Anyscale and get its ID.
         """
-        with open(compute_config_file, "r") as f:
+        with open(compute_config_file) as f:
             config_dict = json.load(f)
         config_object = ComputeTemplateConfig(**config_dict)
         created_template = self.api_client.create_compute_template_api_v2_compute_templates_post(
             create_compute_template=CreateComputeTemplate(
-                name="autogenerated-config-{}".format(datetime.now().isoformat()),
+                name=f"autogenerated-config-{datetime.now().isoformat()}",
                 project_id=project_id,
                 config=config_object,
             )
@@ -438,7 +441,7 @@ class SessionController(BaseController):
         compute_template_id = str(created_template.id)
         return compute_template_id
 
-    def _create_or_update_session_data(
+    def _create_or_update_session_data(  # noqa: PLR0913
         self,
         session_name: str,
         project_id: str,
@@ -532,7 +535,7 @@ class SessionController(BaseController):
                 return str(app_config.id)
 
         raise click.ClickException(
-            "Application config '{}' not found. ".format(app_config_name)
+            f"Application config '{app_config_name}' not found. "
             + "Available app configs: {}".format(", ".join(a.name for a in app_configs))
         )
 
