@@ -4,10 +4,14 @@ from time import sleep, time
 from typing import Any, Dict, Optional, Union
 
 from anyscale.api import configure_tcp_keepalive
-from anyscale.sdk.anyscale_client.models.cluster_compute_config import ClusterComputeConfig
-from anyscale.sdk.anyscale_client.models.create_cluster_compute import CreateClusterCompute
+from anyscale.sdk.anyscale_client.models.cluster_compute_config import (
+    ClusterComputeConfig,
+)
+from anyscale.sdk.anyscale_client.models.create_cluster_compute import (
+    CreateClusterCompute,
+)
 from anyscale.utils.runtime_env import upload_and_rewrite_working_dir
-import anyscale.sdk.anyscale_client as anyscale_client
+from anyscale.sdk import anyscale_client
 from anyscale.sdk.anyscale_client.api.default_api import DefaultApi
 from anyscale.version import __version__ as version
 from anyscale.sdk.anyscale_client.models.create_cluster_environment import (
@@ -27,6 +31,7 @@ from anyscale.authenticate import AuthenticationBlock
 from anyscale_client.models import ClusterEnvironmentBuildStatus
 from anyscale.util import get_ray_and_py_version_for_default_cluster_env
 
+
 def _upload_and_rewrite_working_dir_in_create_production_job(create_production_job):
     if create_production_job.config is not None:
         runtime_env = create_production_job.config.get("runtime_env")
@@ -48,12 +53,19 @@ def _is_create_byod_cluster_environment(obj: Any) -> bool:
         config_json = getattr(obj, "config_json", None)
     return isinstance(config_json, dict) and "docker_image" in config_json
 
+
 # Build states that are in progress
-IN_PROGRESS_BUILD_STATES = [ClusterEnvironmentBuildStatus.IN_PROGRESS, ClusterEnvironmentBuildStatus.PENDING]
+IN_PROGRESS_BUILD_STATES = [
+    ClusterEnvironmentBuildStatus.IN_PROGRESS,
+    ClusterEnvironmentBuildStatus.PENDING,
+]
+
 
 class AnyscaleSDK(DefaultApi):  # type: ignore
     def __init__(
-        self, auth_token: Optional[str] = None, host: Optional[str] = None,
+        self,
+        auth_token: Optional[str] = None,
+        host: Optional[str] = None,
     ):
         # Default host to production but respect the ANYSCALE_HOST environment variable.
         if host is None:
@@ -82,7 +94,9 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
 
     def build_cluster_environment(
         self,
-        create_cluster_environment: Union[CreateClusterEnvironment, CreateBYODClusterEnvironment],
+        create_cluster_environment: Union[
+            CreateClusterEnvironment, CreateBYODClusterEnvironment
+        ],
         poll_rate_seconds: int = 15,
         timeout_seconds: Optional[int] = None,
         log_output: bool = False,
@@ -180,9 +194,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
         timeout = time() + timeout_seconds if timeout_seconds else None
 
         operation = self.get_build(operation_id)
-        url = get_endpoint(
-            f"configurations/app-config-details/{operation_id}"
-        )
+        url = get_endpoint(f"configurations/app-config-details/{operation_id}")
         self.log.info(
             f"Waiting for cluster environment to build. View progress at {url}."
         )
@@ -195,18 +207,14 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
 
             sleep(poll_rate_seconds)
             self.log.info(f"status: {operation.result.status}")
-            operation = self.get_build(
-                operation_id
-            )
+            operation = self.get_build(operation_id)
         if operation.result.status != ClusterEnvironmentBuildStatus.SUCCEEDED:
             raise Exception(
                 f"Failed to build Cluster Environment, you can check the full logs at: {url}."
             )
         else:
             self.log.info("Cluster environment successfully finished building.")
-            return self.get_build(
-                operation_id
-            ).result
+            return self.get_build(operation_id).result
 
     def launch_cluster(
         self,
@@ -259,14 +267,23 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
         else:
             if not cluster_environment_build_id:
                 # Use default cluster environment build when starting cluster without specifying an id
-                ray_version, py_version = get_ray_and_py_version_for_default_cluster_env()
-                cluster_environment_build_id = self.get_default_cluster_environment_build(
-                    f"py{py_version}", ray_version
-                ).result.id
+                (
+                    ray_version,
+                    py_version,
+                ) = get_ray_and_py_version_for_default_cluster_env()
+                cluster_environment_build_id = (
+                    self.get_default_cluster_environment_build(
+                        f"py{py_version}", ray_version
+                    ).result.id
+                )
 
             if not cluster_compute_id:
                 if cluster_compute_config:
-                    cluster_compute_id = self.create_cluster_compute(CreateClusterCompute(config=cluster_compute_config, anonymous=True)).result.id
+                    cluster_compute_id = self.create_cluster_compute(
+                        CreateClusterCompute(
+                            config=cluster_compute_config, anonymous=True
+                        )
+                    ).result.id
                 else:
                     # Use default cluster compute when starting cluster without specifying an id
                     cluster_compute_id = self.get_default_cluster_compute().result.id
@@ -382,34 +399,6 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
         else:
             return self.get_cluster(operation.cluster_id).result
 
-    def fetch_actor_logs(self, actor_id: str) -> str:
-        """
-        Retrieves logs for an Actor.
-        This function may take several minutes if the Cluster this Actor ran on has been terminated.
-
-        Args:
-            actor_id - ID of the Actor
-
-        Returns
-            Log output for the Actor as a string
-
-        Raises
-            Exception if we fail to fetch logs
-        """
-
-        # Timeout in 5 minutes
-        timeout = time() + 300
-
-        logs = self.get_actor_logs(actor_id).result
-        while not logs.ready:
-            if time() > timeout:
-                raise Exception(f"Failed to get logs from Actor {actor_id}")
-
-            sleep(15)
-            logs = self.get_actor_logs(actor_id).result
-
-        return str(logs.logs)
-
     def fetch_job_logs(self, job_id: str) -> str:
         """
         Retrieves logs for a Ray Job or Interactive Session.
