@@ -1,3 +1,4 @@
+# NOTE: The logic in this file/module is only run from within an Anyscale workspace
 import datetime
 import json
 import logging
@@ -27,6 +28,12 @@ CONFIGS_TO_REPLICATE = [
     ".vimrc",
 ]
 
+# Default cluster storage dir.
+CLUSTER_STORAGE_DIR = "/mnt/cluster_storage"
+
+# Default VSCode server dir.
+VSCODE_DESKTOP_SERVER_DIR = "/home/ray/.vscode-server"
+
 # Experimental configs propagated from the cluster config service.
 EFS_IP = os.environ.get("ANYSCALE_EXPERIMENTAL_EFS_IP", "")
 EFS_ROOT = os.environ.get("ANYSCALE_EXPERIMENTAL_EFS_ROOT", "")
@@ -55,10 +62,11 @@ EFS_OBJECTS_DIR = os.environ.get("EFS_OBJECTS_DIR", "/efs/workspaces/shared_obje
 EFS_CREDS_DIR = os.environ.get("EFS_CREDS_DIR", "/efs/generated_credentials")
 RAY_ML_DEV = bool(os.environ.get("RAY_ML_DEV"))
 AUTOGC = bool(os.environ.get("AUTOGC_SNAPSHOTS", True))
+SKIP_VSCODE_DESKTOP_SETUP = bool(os.environ.get("SKIP_VSCODE_DESKTOP_SETUP", False))
 
-# TODO(ekl) enable this flag by default. This causes workspaces to use the $CWD
-# as the runtime env working dir instead of always the workspace root.
-RELATIVE_WORKING_DIR = bool(os.environ.get("RELATIVE_WORKING_DIR", False))
+# This causes workspaces to use the $CWD as the runtime env working dir instead of
+# always the workspace root.
+RELATIVE_WORKING_DIR = bool(os.environ.get("RELATIVE_WORKING_DIR", True))
 
 
 def optimize_git_repo(directory: str, shared_repo: str) -> None:
@@ -75,23 +83,23 @@ def optimize_git_repo(directory: str, shared_repo: str) -> None:
             same repo can share the objects stored in the shared repo.
     """
     start = time.time()
-    objects_path = "{}/.git/objects".format(directory)
+    objects_path = f"{directory}/.git/objects"
     if os.path.exists(objects_path):
         if not os.path.exists(shared_repo):
             os.makedirs(os.path.dirname(shared_repo), exist_ok=True)
             # TODO(ekl) it's faster to do a copy of just the objects dir, but it seems
             # we need to git clone in order for alternates to be recognized as valid.
             subprocess.check_call(  # noqa
-                "git clone --bare {}/ {}/".format(directory, shared_repo), shell=True,
+                f"git clone --bare {directory}/ {shared_repo}/", shell=True,
             )
         shared_objects_dir = os.path.join(shared_repo, "objects")
         subprocess.check_call(  # noqa
-            "rsync -a {}/ {}/".format(objects_path, shared_objects_dir), shell=True
+            f"rsync -a {objects_path}/ {shared_objects_dir}/", shell=True
         )
         subprocess.check_call("rm -rf {}".format(objects_path), shell=True)  # noqa
         os.makedirs(os.path.join(objects_path, "info"), exist_ok=True)
         with open(os.path.join(objects_path, "info/alternates"), "w") as f:
-            f.write("{}\n".format(shared_objects_dir))
+            f.write(f"{shared_objects_dir}\n")
     logger.info(
         "Synced git objects for {} to {} in {}s.".format(
             directory, shared_repo, time.time() - start
@@ -169,7 +177,7 @@ def unpack_snapshot_zip(zip_path: str, directory: str) -> None:
     start = time.time()
     os.makedirs(directory, exist_ok=True)
     subprocess.check_call(  # noqa
-        "unzip -X -o -q {} -d {}".format(zip_path, directory), shell=True
+        f"unzip -X -o -q {zip_path} -d {directory}", shell=True
     )
     logger.info(
         "Unpacked snapshot {} to {} in {}s.".format(
@@ -181,7 +189,7 @@ def unpack_snapshot_zip(zip_path: str, directory: str) -> None:
 def compute_content_hash(zip_path: str) -> bytes:
     """Return the md5 hash of a given zipfile on disk."""
     md5 = subprocess.check_output(  # noqa
-        "unzip -p {} | md5sum -b | cut -f1 -d ' '".format(zip_path), shell=True
+        f"unzip -p {zip_path} | md5sum -b | cut -f1 -d ' '", shell=True
     )
     md5 = md5.strip()
     return md5
@@ -200,13 +208,14 @@ def get_or_create_snapshot_zip(directory: str, auto: bool) -> str:
     old_zip = find_latest(ignore_base_snapshot=auto)
     if old_zip:
         try:
-            old_hash: Optional[bytes] = open(old_zip + ".md5", "rb").read().strip()
-        except Exception:
+            with open(old_zip + ".md5", "rb") as f:
+                old_hash: Optional[bytes] = f.read().strip()
+        except Exception:  # noqa: BLE001
             logger.warning("Failed to read md5 file")
             old_hash = None
     else:
         old_hash = None
-    logger.info("Content hashes {!r} vs {!r}".format(old_hash, new_hash))
+    logger.info(f"Content hashes {old_hash!r} vs {new_hash!r}")
     if old_hash == new_hash:
         logger.info("Content hash unchanged, not saving new snapshot.")
         os.unlink(new_zip)
@@ -236,7 +245,7 @@ def do_snapshot(auto: bool = False):
 
         # TODO(ekl) should we isolate the objects by workspace or repo?
         optimize_git_repo(WORKING_DIR, EFS_OBJECTS_DIR)
-        zip = get_or_create_snapshot_zip(WORKING_DIR, auto)
+        zip = get_or_create_snapshot_zip(WORKING_DIR, auto)  # noqa: A001
 
         # If the zip was already on EFS, we're done.
         if zip.startswith(snapshot_dir):
@@ -256,7 +265,7 @@ def do_snapshot(auto: bool = False):
         # report successful idle termination status
         report_idle_termination_activity()
 
-    except Exception:
+    except Exception:  # noqa: BLE001
         logger.exception("Failed to create a snapshot")
         # report error idle termination status
         report_idle_termination_error("Failed to create a snapshot")
@@ -300,17 +309,17 @@ def gc_snapshots() -> None:
     # Never GC the latest snapshot.
     snapshots = snapshots[:-1]
     horizon = datetime.datetime.now(datetime.timezone.utc) - datetime.timedelta(hours=1)
-    prefix = "snapshot_{}_".format(horizon.isoformat())
+    prefix = f"snapshot_{horizon.isoformat()}_"
     # GC snapshots older than 1 hour.
     for snapshot in snapshots:
         if snapshot < prefix:
             full_path = os.path.join(snapshot_dir, snapshot)
             md5 = full_path + ".md5"
-            logger.info("Deleting old snapshot {}".format(full_path))
+            logger.info(f"Deleting old snapshot {full_path}")
             try:
                 os.unlink(full_path)
                 os.unlink(md5)
-            except Exception:
+            except Exception:  # noqa: BLE001
                 logger.exception("Failed to delete snapshot")
 
 
@@ -329,12 +338,12 @@ def find_latest(ignore_base_snapshot: bool) -> Optional[str]:
     return os.path.join(snapshot_dir, snapshots[-1])
 
 
-def find_base_snapshot() -> Optional[str]:
+def find_base_snapshot() -> Optional[str]:  # noqa: PLR0911
     if not BASE_SNAPSHOT:
         return None
     try:
         base_data = json.loads(BASE_SNAPSHOT)
-    except Exception:
+    except Exception:  # noqa: BLE001
         logger.exception("Failed to parse base snapshot info")
         return None
 
@@ -356,7 +365,7 @@ def find_base_snapshot() -> Optional[str]:
             return None
         logger.info(f"Base snapshot from workspace {workspace_id} and {iso_time}")
         snapshot_dir = os.path.join(EFS_WORKSPACE_DIR, workspace_id, "snapshots")
-        snapshot_time = "snapshot_{}_".format(iso_time)
+        snapshot_time = f"snapshot_{iso_time}_"
         if not os.path.exists(snapshot_dir):
             return None
         # Find the latest matching snapshot before the time stamp
@@ -529,7 +538,7 @@ def env_hook(runtime_env) -> Dict[str, Any]:
     # https://github.com/anyscale/product/issues/11679
     if env_vars:
         runtime_env["env_vars"] = env_vars
-    logger.info("Updated runtime env to {}".format(runtime_env))
+    logger.info(f"Updated runtime env to {runtime_env}")
     return runtime_env
 
 
@@ -566,7 +575,7 @@ def setup_credentials():
     for cmd in setup_ssh_directory_cmds:
         try:
             subprocess.check_call(cmd, shell=True)  # noqa
-        except Exception:
+        except Exception:  # noqa: BLE001
             logger.exception(f"Error running {cmd}")
 
     if os.path.exists(private_key):
@@ -595,9 +604,9 @@ def _load_env_vars():
     env_vars = {}
     if os.path.exists(env_store):
         try:
-            with open(env_store, "r") as f:
+            with open(env_store) as f:
                 env_vars = json.loads(f.read())
-        except Exception:
+        except Exception:  # noqa: BLE001
             logger.exception("Error parsing env vars")
     return env_vars
 
@@ -634,16 +643,12 @@ def setup_nfs():
     EFS_MOUNT = f"sudo mount -t nfs4 -o nfsvers=4.1,rsize=1048576,wsize=1048576,hard,timeo=600,retrans=2,noresvport {EFS_IP}:/ /efs"
     FILE_STORE_MOUNT = f"sudo mount -o rsize=1048576,wsize=1048576,hard,timeo=600,retrans=2,nolock {EFS_IP}:/{EFS_ROOT} /efs"
     COMMANDS = [
-        # TODO: move dep install to base image
-        "sudo apt-get update",
-        "sudo apt-get install -y nfs-common zip unzip awscli",
         "sudo mkdir -p /efs",
         EFS_MOUNT if EFS_TYPE == "EFS" else FILE_STORE_MOUNT,
         "sudo mkdir -p /efs",
         "sudo chown ray /efs",
         f"mkdir -p /efs/workspaces/{WORKSPACE_ID}/cluster_storage",
-        f"sudo ln -sf /efs/workspaces/{WORKSPACE_ID}/cluster_storage "
-        "/mnt/cluster_storage",
+        f"sudo ln -sf /efs/workspaces/{WORKSPACE_ID}/cluster_storage {CLUSTER_STORAGE_DIR}",
         f"mkdir -p /efs/users/{USERNAME}",
         f"sudo ln -sf /efs/users/{USERNAME} /mnt/user_storage",
         "mkdir -p /efs/shared_storage",
@@ -654,7 +659,7 @@ def setup_nfs():
     for cmd in COMMANDS:
         try:
             subprocess.check_call(cmd, shell=True)  # noqa
-        except Exception:
+        except Exception:  # noqa: BLE001
             logger.exception(f"Error running {cmd}")
 
     # on first load, report an activity to signal healthy start
@@ -662,8 +667,13 @@ def setup_nfs():
 
 
 def setup_vscode_desktop():
-    local_folder = "/home/ray/.vscode-server"
-    remote_folder = "/mnt/cluster_storage/vscode_desktop"
+    """Setup vscode desktop extension directory."""
+
+    if SKIP_VSCODE_DESKTOP_SETUP:
+        return
+
+    local_folder = VSCODE_DESKTOP_SERVER_DIR
+    remote_folder = os.path.join(CLUSTER_STORAGE_DIR, "vscode_desktop")
 
     # create the local folder
     os.makedirs(local_folder, exist_ok=True)
@@ -676,7 +686,7 @@ def setup_vscode_desktop():
         os.symlink(directory, f"{local_folder}/{folder}")
 
 
-def setup_container(ray_params: Any, is_head: bool):
+def setup_container(ray_params: Any, is_head: bool):  # noqa: ARG001, PLR0912
     """Setup the container synchronously prior to Ray start.
 
     This handles (1) mounting network storage, (2) restoring workspace data, and
@@ -691,7 +701,7 @@ def setup_container(ray_params: Any, is_head: bool):
     else:
         try:
             setup_nfs()
-        except Exception:
+        except Exception:  # noqa: BLE001
             logger.exception("Failed to setup NFS")
         if not WORKSPACE_ID:
             logger.info("No workspace id configured, skipping workspace restore.")
@@ -700,15 +710,15 @@ def setup_container(ray_params: Any, is_head: bool):
         else:
             try:
                 restore_latest()
-            except Exception:
+            except Exception:  # noqa: BLE001
                 logger.exception("Failed to restore workspace")
             try:
                 setup_credentials()
-            except Exception:
+            except Exception:  # noqa: BLE001
                 logger.exception("Failed to setup SSH credentials")
             try:
                 setup_vscode_desktop()
-            except Exception:
+            except Exception:  # noqa: BLE001
                 logger.exception("Failed to setup VSCode desktop")
     with open("/tmp/initialized", "w") as f:
         f.write("ok")
@@ -718,7 +728,7 @@ def autosnapshot_loop():
     if not WORKSPACE_ID:
         logger.info("Workspaces disabled.")
         return
-    logger.info("Started autosnapshot loop with interval {}".format(SNAPSHOT_INTERVAL))
+    logger.info(f"Started autosnapshot loop with interval {SNAPSHOT_INTERVAL}")
     while True:
         time.sleep(SNAPSHOT_INTERVAL)
         do_snapshot(auto=True)
@@ -730,7 +740,7 @@ def post_build():
         # TODO: remove this after next image release.
         "echo 'PROMPT_COMMAND=\"history -a\"' >> /home/ray/.bashrc",
         "echo '[ -e ~/.workspacerc ] && source ~/.workspacerc' >> /home/ray/.bashrc",
-        "echo 'export PATH=/mnt/cluster_storage/pypi/bin:$PATH' >> /home/ray/.bashrc",
+        f"echo 'export PATH={CLUSTER_STORAGE_DIR}/pypi/bin:$PATH' >> /home/ray/.bashrc",
     ]
     for cmd in CMDS:
         print("Executing", cmd)
