@@ -1,8 +1,9 @@
 from typing import Any, Dict, Iterator, List, Optional, Tuple
 from unittest.mock import ANY, MagicMock, Mock, patch
 
-import botocore
+import boto3
 from click import ClickException
+from moto import mock_cloudformation
 import pytest
 
 from anyscale.aws_iam_policies import (
@@ -273,7 +274,7 @@ def test_setup_cloud_gcp_vpc_peering(
 
 
 @pytest.mark.parametrize(
-    "vpc_peering_options,error_output",
+    ("vpc_peering_options", "error_output"),
     [
         (
             ("not_an_ip_address", "project_id", "vpc_id"),
@@ -375,7 +376,7 @@ def test_delete_cloud_by_id(cloud_test_data: Cloud) -> None:
 
 
 @pytest.mark.parametrize(
-    "role_arn,should_delete",
+    ("role_arn", "should_delete"),
     [
         pytest.param(f"arn:aws:iam::012345678901:role/{ANYSCALE_IAM_ROLE_NAME}", True),
         pytest.param(
@@ -470,7 +471,6 @@ def test_prepare_for_managed_cloud_setup() -> None:
     mock_create_cloud_api_v2_clouds_post = Mock(
         return_value=Mock(result=Mock(id="mock_cloud_id"))
     )
-    mock_setup_aws_ray_role = Mock(return_value=Mock(arn="mock_ray_iam_role_arn"))
 
     with patch.multiple(
         "anyscale.controllers.cloud_controller",
@@ -479,19 +479,16 @@ def test_prepare_for_managed_cloud_setup() -> None:
         get_user_env_aws_account=mock_get_user_env_aws_account,
     ), patch.multiple("secrets", token_hex=Mock(return_value="02e38860")):
         cloud_controller = CloudController()
-        cloud_controller.setup_aws_ray_role = mock_setup_aws_ray_role  # type: ignore
         cloud_controller.api_client.create_cloud_api_v2_clouds_post = (
             mock_create_cloud_api_v2_clouds_post
         )
 
         (
             anyscale_iam_role_name,
-            ray_iam_role_arn,
             created_cloud_id,
         ) = cloud_controller.prepare_for_managed_cloud_setup("us-west-2", "cloud_name")
 
     assert anyscale_iam_role_name == "anyscale-iam-role-02e38860"
-    assert ray_iam_role_arn == "mock_ray_iam_role_arn"
     assert created_cloud_id == "mock_cloud_id"
 
     mock_get_available_regions.assert_called_once_with()
@@ -506,9 +503,6 @@ def test_prepare_for_managed_cloud_setup() -> None:
             is_bring_your_own_resource=False,
         )
     )
-    mock_setup_aws_ray_role.assert_called_once_with(
-        "us-west-2", f"{created_cloud_id}-cluster_node_role"
-    )
 
 
 @pytest.mark.parametrize(
@@ -541,7 +535,7 @@ def test_setup_aws_ray_role(roles: List[Optional[Mock]]) -> None:
     roles[-1].attach_policy.assert_any_call(
         PolicyArn=AMAZON_ECR_READONLY_ACCESS_POLICY_ARN
     )
-    assert 2 == roles[-1].attach_policy.call_count
+    assert roles[-1].attach_policy.call_count == 2
     mock_iam.create_instance_profile.assert_called_once_with(
         InstanceProfileName="ray-autoscaler-v1"
     )
@@ -649,14 +643,22 @@ def test_register_cloud(
     mock_security_group_ids = ["security_group_id"]
     mock_s3_bucket_id = "mock_s3_bucket_id"
     mock_get_role = Mock()
+    mock_get_aws_efs_mount_target_ip = Mock(return_value="mock_efs_ip")
+
+    def mock_associate_subnet_ids_with_policy(resource, region):
+        if verify_success:
+            return resource
+        raise RuntimeError
 
     with patch(
         "anyscale.controllers.cloud_controller.CloudController.verify_aws_cloud_resources",
         new=Mock(return_value=verify_success),
     ), patch.multiple(
         "anyscale.controllers.cloud_controller",
+        associate_subnets_with_azs=mock_associate_subnet_ids_with_policy,
         _update_external_ids_for_policy=Mock(return_value={"mock_policy": "test"}),
         _get_role=mock_get_role,
+        _get_aws_efs_mount_target_ip=mock_get_aws_efs_mount_target_ip,
     ):
         cloud_controller = CloudController()
         cloud_controller.api_client = mock_api_client
@@ -675,7 +677,7 @@ def test_register_cloud(
                 private_network=private_network,
             )
         else:
-            with pytest.raises(ClickException):
+            with pytest.raises(ClickException) as e:
                 cloud_controller.register_aws_cloud(
                     region=mock_region,
                     name=mock_cloud_name,
@@ -688,6 +690,7 @@ def test_register_cloud(
                     s3_bucket_id=mock_s3_bucket_id,
                     private_network=private_network,
                 )
+            e.match("Please make sure all the resources provided")
     mock_get_role.assert_called_once_with("mock_anyscale_role", mock_region)
 
     mock_api_client.create_cloud_api_v2_clouds_post.assert_called_with(
@@ -712,6 +715,7 @@ def test_register_cloud(
                     aws_security_groups=mock_security_group_ids,
                     aws_s3_id=mock_s3_bucket_id,
                     aws_efs_id=mock_efs_id,
+                    aws_efs_mount_target_ip="mock_efs_ip",
                 ),
             ),
         )
@@ -733,7 +737,7 @@ def test_verify_cloud(
 ):
     cloud_resource_mock = Mock() if has_cloud_resource else None
     verify_mock = Mock(return_value=True)
-    verify_aws_subnets_mock = Mock(return_value=(True, cloud_resource_mock))
+    verify_aws_subnets_mock = Mock(return_value=True)
     with patch.multiple(
         "anyscale.controllers.cloud_controller",
         get_cloud_id_and_name=Mock(return_value=("", "")),
@@ -797,11 +801,11 @@ def test_setup_managed_cloud(cloud_setup_failure) -> None:
     mock_cloud_name = "mock_cloud_name"
     mock_cloud_id = "mock_cloud_id"
     mock_anyscale_iam_role_name = "mock_anyscale_iam_role_name"
-    mock_ray_iam_role_arn = "mock_ray_iam_role_arn"
+    mock_ray_iam_role_name = f"{mock_cloud_id}-cluster_node_role"
     mock_cfn_stack = "mock_cfn_stack"
     mock_api_client = Mock()
     mock_prepare_for_managed_cloud_setup = Mock(
-        return_value=(mock_anyscale_iam_role_name, mock_ray_iam_role_arn, mock_cloud_id)
+        return_value=(mock_anyscale_iam_role_name, mock_cloud_id)
     )
     mock_run_cloudformation = Mock(return_value=mock_cfn_stack)
     mock_update_cloud_with_resources = Mock()
@@ -831,11 +835,9 @@ def test_setup_managed_cloud(cloud_setup_failure) -> None:
         mock_region, mock_cloud_name
     )
     mock_run_cloudformation.assert_called_with(
-        mock_region, mock_cloud_id, mock_anyscale_iam_role_name, mock_ray_iam_role_arn
-    )
-    mock_update_cloud_with_resources.assert_called_with(
-        mock_cfn_stack, mock_cloud_id, mock_ray_iam_role_arn
+        mock_region, mock_cloud_id, mock_anyscale_iam_role_name, mock_ray_iam_role_name
     )
+    mock_update_cloud_with_resources.assert_called_with(mock_cfn_stack, mock_cloud_id)
 
     if cloud_setup_failure:
         mock_api_client.delete_cloud_api_v2_clouds_cloud_id_delete.assert_called_with(
@@ -883,36 +885,25 @@ def test_delete_cloud():
         )
 
 
-def test_delete_aws_managed_cloud():
-    fake_stack = {
-        "StackId": "arn:aws:cloudformation:us-west-2:012345678910:stack/mock_stack_id/more_random_id",
-        "StackStatus": "DELETE_COMPLETE",
-    }
-    mock_client = Mock(
-        describe_stacks=Mock(
-            side_effect=[
-                {"Stacks": [fake_stack]},
-                botocore.exceptions.ClientError(
-                    {"Error": {"Code": "MyCode", "Message": "MyMessage"}}, "CreateNode"
-                ),
-            ]
-        ),
-        delete_stack=Mock(),
-        list_stacks=Mock(return_value={"StackSummaries": [fake_stack]}),
+@pytest.mark.parametrize("pre_deleted", [True, False])
+@mock_cloudformation
+def test_delete_aws_managed_cloud(pre_deleted: bool):
+    cfn = boto3.client("cloudformation", region_name="us-west-2")
+    stack_response = cfn.create_stack(
+        StackName="stack_name", TemplateBody="{Resources: {}}"
     )
-    with patch.multiple(
-        "anyscale.controllers.cloud_controller", _client=Mock(return_value=mock_client),
-    ):
-        cloud_mock = Mock(
-            id="mock_cloud_id",
-            region="mock_region",
-            cloud_resource=Mock(aws_cloudformation_stack_id=fake_stack["StackId"],),
-        )
-        cloud_controller = CloudController()
-        mock_log_spinner = MagicMock()
-        mock_log_spinner.return_value.__enter__.return_value = Mock()
-        cloud_controller.log = Mock(spinner=mock_log_spinner)
+    stack_id = stack_response["StackId"]
+    if pre_deleted:
+        cfn.delete_stack(StackName=stack_id)
 
-        delete_result = cloud_controller.delete_aws_managed_cloud(cloud=cloud_mock)
+    cloud_mock = Mock(
+        id="mock_cloud_id",
+        region="us-west-2",
+        cloud_resource=Mock(aws_cloudformation_stack_id=stack_id,),
+    )
+    cloud_controller = CloudController()
+    mock_log_spinner = MagicMock()
+    mock_log_spinner.return_value.__enter__.return_value = Mock()
+    cloud_controller.log = Mock(spinner=mock_log_spinner)
 
-        assert delete_result
+    assert cloud_controller.delete_aws_managed_cloud(cloud=cloud_mock)
