@@ -525,7 +525,7 @@ def test_anyscale_start(
 @pytest.mark.parametrize("on_head_node", [True, False])
 @pytest.mark.parametrize("worker_node_id", [None, "123456"])
 @pytest.mark.parametrize("worker_node_ip", [None, "0.0.0.0"])
-def test_ssh_session(
+def test_ssh_session(  # noqa: PLR0913
     session_test_data: Session,
     mock_auth_api_client_multiple_sessions,
     on_head_node: bool,
